@@ -32,7 +32,6 @@
 namespace geos {
 namespace geom { // geos::geom
 class Coordinate;
-class CoordinateArraySequence;
 class CoordinateSequenceFilter;
 }
 }
@@ -109,11 +108,17 @@ public:
      */
     Dimension::DimensionType getDimension() const override;
 
+    bool hasDimension(Dimension::DimensionType d) const override;
+
     bool isDimensionStrict(Dimension::DimensionType d) const override;
 
     /// Returns coordinate dimension.
     uint8_t getCoordinateDimension() const override;
 
+    bool hasM() const override;
+
+    bool hasZ() const override;
+
     std::unique_ptr<Geometry> getBoundary() const override;
 
     /**
@@ -132,6 +137,8 @@ public:
     bool equalsExact(const Geometry* other,
                      double tolerance = 0) const override;
 
+    bool equalsIdentical(const Geometry* other) const override;
+
     void apply_ro(CoordinateFilter* filter) const override;
 
     void apply_rw(const CoordinateFilter* filter) override;
@@ -150,7 +157,7 @@ public:
 
     void normalize() override;
 
-    const Coordinate* getCoordinate() const override;
+    const CoordinateXY* getCoordinate() const override;
 
     /// Returns the total area of this collection
     double getArea() const override;
@@ -182,9 +189,17 @@ public:
      */
     std::unique_ptr<GeometryCollection> reverse() const { return std::unique_ptr<GeometryCollection>(reverseImpl()); }
 
+    const Envelope* getEnvelopeInternal() const override {
+        if (envelope.isNull()) {
+            envelope = computeEnvelopeInternal();
+        }
+        return &envelope;
+    }
+
 protected:
 
     GeometryCollection(const GeometryCollection& gc);
+    GeometryCollection& operator=(const GeometryCollection& gc);
 
     /** \brief
      * Construct a GeometryCollection with the given GeometryFactory.
@@ -201,17 +216,8 @@ protected:
      *	Elements may be empty <code>Geometry</code>s,
      *	but not <code>null</code>s.
      *
-     *	If construction succeed the created object will take
-     *	ownership of newGeoms vector and elements.
-     *
-     *	If construction	fails "IllegalArgumentException *"
-     *	is thrown and it is your responsibility to delete newGeoms
-     *	vector and content.
-     *
      * @param newFactory the GeometryFactory used to create this geometry
      */
-    GeometryCollection(std::vector<Geometry*>* newGeoms, const GeometryFactory* newFactory);
-
     GeometryCollection(std::vector<std::unique_ptr<Geometry>> && newGeoms, const GeometryFactory& newFactory);
 
     /// Convenience constructor to build a GeometryCollection from vector of Geometry subclass pointers
@@ -230,11 +236,19 @@ protected:
     };
 
     std::vector<std::unique_ptr<Geometry>> geometries;
+    mutable Envelope envelope;
 
-    Envelope::Ptr computeEnvelopeInternal() const override;
+    Envelope computeEnvelopeInternal() const;
+
+    void geometryChangedAction() override {
+        envelope.setToNull();
+    }
 
     int compareToSameClass(const Geometry* gc) const override;
 
+    bool hasCurvedComponents() const override;
+
+
 };
 
 } // namespace geos::geom
