@@ -19,15 +19,12 @@
  **********************************************************************/
 
 
-#ifndef GEOS_GEOMGRAPH_EDGEINTERSECTION_H
-#define GEOS_GEOMGRAPH_EDGEINTERSECTION_H
+#pragma once
 
 #include <geos/export.h>
 
 #include <geos/geom/Coordinate.h> // for composition and inlines
 
-#include <geos/inline.h>
-
 #include <ostream>
 
 
@@ -42,7 +39,7 @@ namespace geomgraph { // geos.geomgraph
  * The intersection point must be precise.
  *
  */
-class GEOS_DLL EdgeIntersection {
+class GEOS_DLL EdgeIntersection final {
 public:
 
     // the point of intersection
@@ -150,6 +147,3 @@ operator<< (std::ostream& os, const EdgeIntersection& e)
 } // namespace geos.geomgraph
 } // namespace geos
 
-#endif // ifndef GEOS_GEOMGRAPH_EDGEINTERSECTION_H
-
-
