@@ -18,8 +18,7 @@
  **********************************************************************/
 
 
-#ifndef GEOS_GEOMGRAPH_EDGELIST_H
-#define GEOS_GEOMGRAPH_EDGELIST_H
+#pragma once
 
 #include <geos/export.h>
 #include <vector>
@@ -29,8 +28,6 @@
 
 #include <geos/noding/OrientedCoordinateArray.h> // for map comparator
 
-#include <geos/inline.h>
-
 #ifdef _MSC_VER
 #pragma warning(push)
 #pragma warning(disable: 4251) // warning C4251: needs to have dll-interface to be used by clients of class
@@ -55,7 +52,7 @@ namespace geomgraph { // geos.geomgraph
  * It supports locating edges
  * that are pointwise equals to a target edge.
  */
-class GEOS_DLL EdgeList {
+class GEOS_DLL EdgeList final {
 
 private:
 
@@ -88,7 +85,7 @@ public:
         ocaMap()
     {}
 
-    virtual ~EdgeList() = default;
+    ~EdgeList() = default;
 
     /**
      * Insert an edge unless it is already in the list
@@ -121,12 +118,7 @@ std::ostream& operator<< (std::ostream& os, const EdgeList& el);
 } // namespace geos.geomgraph
 } // namespace geos
 
-//#ifdef GEOS_INLINE
-//# include "geos/geomgraph/EdgeList.inl"
-//#endif
-
 #ifdef _MSC_VER
 #pragma warning(pop)
 #endif
 
-#endif // ifndef GEOS_GEOMGRAPH_EDGELIST_H
