@@ -14,7 +14,7 @@
  *
  **********************************************************************
  *
- * Last port: operation/buffer/BufferBuilder.java r378 (JTS-1.12)
+ * Last port: operation/buffer/BufferBuilder.java 149b38907 (JTS-1.12)
  *
  **********************************************************************/
 
@@ -56,10 +56,9 @@ class PlanarGraph;
 namespace operation {
 namespace buffer {
 class BufferSubgraph;
-}
-namespace overlay {
 class PolygonBuilder;
 }
+
 }
 }
 
@@ -233,7 +232,7 @@ private:
      *        the final polygons
      */
     void buildSubgraphs(const std::vector<BufferSubgraph*>& subgraphList,
-                        overlay::PolygonBuilder& polyBuilder);
+                        PolygonBuilder& polyBuilder);
 
     /// \brief
     /// Return the externally-set noding::Noder OR a newly created
