@@ -16,8 +16,7 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_OP_BUFFER_BUFFERSUBGRAPH_H
-#define GEOS_OP_BUFFER_BUFFERSUBGRAPH_H
+#pragma once
 
 #include <geos/export.h>
 
@@ -80,7 +79,7 @@ private:
     void addReachable(geomgraph::Node* startNode);
 
     /// Adds the argument node and all its out edges to the subgraph
-    //
+    ///
     /// @param node the node to add
     /// @param nodeStack the current set of nodes being traversed
     ///
@@ -201,5 +200,3 @@ bool BufferSubgraphGT(BufferSubgraph* first, BufferSubgraph* second);
 #pragma warning(pop)
 #endif
 
-#endif // ndef GEOS_OP_BUFFER_BUFFERSUBGRAPH_H
-
