@@ -16,12 +16,11 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_TRIANGULATE_QUADEDGE_LOCATEFAILUREEXCEPTION_H
-#define GEOS_TRIANGULATE_QUADEDGE_LOCATEFAILUREEXCEPTION_H
+#pragma once
 
 #include <string>
 
-#include <geos/util.h>
+#include <geos/util/GEOSException.h>
 
 namespace geos {
 namespace triangulate { //geos.triangulate
@@ -36,5 +35,3 @@ public:
 } //namespace geos.triangulate
 } //namespace goes
 
-#endif //GEOS_TRIANGULATE_QUADEDGE_LOCATEFAILUREEXCEPTION_H
-
